#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <unistd.h>
#include <limits.h>

#include "configurator.h"
#include "../config-path.h"

/* 설정 데이터. 렌더링 설정은 기본값을 일단 삽입  */
static s_confdata Conf = { .exec_path = NULL, 
							.rompath = NULL, .tmp_path = NULL };
static s_rendconf Rend = { .res_x = 640, .res_y = 480,
					.fullscreen = 0, .color_depth = 16,
					.scanlines = 0, .filtering = 0,
					.blending = 0, .dithering = 0,
					.showfps = 0, .frame_limit = 1,
					.frame_skip = 0, .frame_det = 1,
					.framerate = 100, .tex_type = 3,
					.tex_cache = 2, .turn_disp = 0 };
static s_windowconf WinInfo = { .root_x = 10, .root_y = 10, 
								.width = 640, .height = 480 };

/* 메인 설정: 키-값 설정 */
short int config_keyval_set (const gchar *key, const gchar *value)
{
	if (0 == g_ascii_strncasecmp (key, "rompath_dir", 11))
	{		
		g_free (Conf.rompath);
		Conf.rompath = NULL;
		Conf.rompath = g_strndup (value, 400);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "exec_dir", 8))
	{
		g_free (Conf.exec_path);
		Conf.exec_path = NULL;
		Conf.exec_path = g_strndup (value, 400);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "tmp_dir", 7))
	{
		g_free (Conf.tmp_path);
		Conf.tmp_path = NULL;
		Conf.tmp_path = g_strndup (value, 400);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "gzinc_root_x", 12))
	{
		WinInfo.root_x = atoi ((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "gzinc_root_y", 12))
	{
		WinInfo.root_y = atoi ((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "gzinc_width", 11))
	{
		WinInfo.width = atoi ((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "gzinc_height", 12))
	{
		WinInfo.height = atoi ((const char *) value);
		return 0;
	}

	return -1;
}

/* 렌더러 설정: 키-값 설정 */
short int rend_conf_keyval_set (const gchar *key, const gchar *value)
{
	if (0 == g_ascii_strncasecmp (key, "XSize", 5))
	{		
		Rend.res_x = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "YSize", 5))
	{
		Rend.res_y = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "FullScreen", 10))
	{
		Rend.fullscreen = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "ColorDepth", 10))
	{
		Rend.color_depth = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "ScanLines", 9))
	{
		Rend.scanlines = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "Filtering", 9))
	{
		Rend.filtering = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "Blending", 8))
	{
		Rend.blending = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "Dithering", 9))
	{
		Rend.dithering = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "ShowFPS", 7))
	{
		Rend.showfps = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "FrameLimitation", 15))
	{
		Rend.frame_limit = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "FrameSkipping", 13))
	{
		Rend.frame_skip = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "FramerateDetection", 18))
	{
		Rend.frame_det = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "FramerateManual", 15))
	{
		Rend.framerate = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "TextureType", 11))
	{
		Rend.tex_type = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "TextureCaching", 14))
	{
		Rend.tex_cache = atoi((const char *) value);
		return 0;
	}
	else if (0 == g_ascii_strncasecmp (key, "TurnDisplay", 11))
	{
		Rend.turn_disp = atoi((const char *) value);
		return 0;
	}

	return -1;
}


/*****************************************
 * Renderer 설정 파일 관련 함수 구현부   *
 *****************************************/
static short int rend_conf_parse (FILE *cfile)
{
	char *buf = NULL;
	gchar **option_set = NULL;

	buf = (char *) malloc (sizeof(char) * SHRT_MAX);
	if (buf != NULL)
	{
		while (NULL != fgets (buf, sizeof(char) * SHRT_MAX, cfile))
		{
			option_set = g_strsplit (buf, "=", 2);
			/* 설정 파일의 매 줄 앞 공백과 뒷 공백, 그리고 = 사이의 공백
			 * 을 이상없이 구분하기 위해, 각 문자열을 strip 하여 넘긴다 */
			if (0 != rend_conf_keyval_set (g_strstrip (option_set[0]), g_strstrip (option_set[1])))
			{
				fprintf (stderr, ".gzinc_rend : invalid configuration line found, ignore.\n");
			}
			g_strfreev (option_set);
		}
	}
	else
	{
		// buffer m-allocation failure
		return -1;
	}

	free (buf);
	return 0;
}

/** @brief 설정 스캐너. 내부에서만 사용.
 */
static short int config_parse (FILE *cfile)
{
	char *buf = NULL;
	gchar **option_set = NULL;

	buf = (char *) malloc (sizeof(char) * SHRT_MAX);
	if (buf != NULL)
	{
		while (NULL != fgets (buf, sizeof(char) * SHRT_MAX, cfile))
		{
			option_set = g_strsplit (buf, "=", 2);
			/* 설정 파일의 매 줄 앞 공백과 뒷 공백, 그리고 = 사이의 공백
			 * 을 이상없이 구분하기 위해, 각 문자열을 strip 하여 넘긴다 */
			if (0 != config_keyval_set (g_strstrip (option_set[0]), g_strstrip (option_set[1])))
			{
				fprintf (stderr, ".gzinc : invalid configuration line found, ignore.\n");
			}
			g_strfreev (option_set);
		}
	}
	else
	{
		// buffer m-allocation failure
		return -1;
	}

	free (buf);
	return 0;
}


/*********************************
 * 공용 함수 구현부              *
 *********************************/

/** @brief	설정 파일을 읽는 시작부
 */
short int config_read (const char *filename, int type)
{
	short int result = 0;
	gchar *cfile_name = NULL;	/**< 설정 파일 이름을 담는 공간. */
	FILE *config_file = NULL;	/**< 설정 파일 포인터 */
	
	cfile_name = g_strdup_printf ("%s/%s", getenv ("HOME"), filename);
	
	if (0 == g_access (cfile_name, R_OK)) 
	{
		config_file = (FILE *) fopen (cfile_name, "rb");
		/* config scanner 로 돌려준다 */
#ifdef DEBUG
		fprintf (stderr, "%s configuration found.\n", cfile_name);
#endif
		if (1 == type)
			result = config_parse (config_file);
		else if (2 == type)
			result = rend_conf_parse (config_file);

		fclose (config_file);
	}
	else
	{
#ifdef DEBUG
		fprintf (stderr, "%s configuration NOT found.\n", cfile_name);
#endif
		result = -2;
		/* 설정 어시스턴트를 실행한다 */
		// result = config_wizard_assistant ();
	}

	g_free (cfile_name);

	return result;
}

/*********************************
 * 일반 설정용 함수 구현부       *
 *********************************/

short int config_write (void)
{
	short int result = 0;
	gchar *cfile_name = NULL;	/**< 설정 파일 이름을 담는 공간. */
	FILE *config_file = NULL;	/**< 설정 파일 포인터 */
	
	cfile_name = g_strdup_printf ("%s/.gzinc", getenv ("HOME"));
	config_file = (FILE *) g_fopen (cfile_name, "w+");

	if (NULL != config_file)
	{
		/* config scanner 로 돌려준다 */
#ifdef DEBUG
		fprintf (stderr, "configuration writing...\n");
#endif
		if (NULL != Conf.rompath)
			fprintf (config_file, "rompath_dir = %s\n", Conf.rompath);
		if (NULL != Conf.exec_path)
			fprintf (config_file, "exec_dir = %s\n", Conf.exec_path);
		if (NULL != Conf.tmp_path)
			fprintf (config_file, "tmp_dir = %s\n", Conf.tmp_path);

		/* 현재 윈도우 포지션, 크기 등을 저장한다 */
		fprintf (config_file, "gzinc_root_x = %d\n", WinInfo.root_x);
		fprintf (config_file, "gzinc_root_y = %d\n", WinInfo.root_y);
		fprintf (config_file, "gzinc_width = %d\n", WinInfo.width);
		fprintf (config_file, "gzinc_height = %d\n", WinInfo.height);
		
		// 쓰기 완료, 닫아준다
		g_free (cfile_name);
		fclose (config_file);
	}
	else
	{
#ifdef DEBUG
		fprintf (stderr, "%s writing failure. Permission?\n", cfile_name);
#endif
		result = -2;
	}

	return result;

}

/* 렌더러 설정 쓰기 함수 */
short int rend_conf_write (void)
{
	short int result = 0;
	gchar *cfile_name = NULL;	/**< 설정 파일 이름을 담는 공간. */
	FILE *config_file = NULL;	/**< 설정 파일 포인터 */
	
	cfile_name = g_strdup_printf ("%s/.gzinc_rend", getenv ("HOME"));
	config_file = (FILE *) g_fopen (cfile_name, "w+");

	if (NULL != config_file)
	{
		/* config scanner 로 돌려준다 */
#ifdef DEBUG
		fprintf (stderr, "configuration writing...\n");
#endif
		fprintf (config_file, "XSize = %d\n", Rend.res_x);
		fprintf (config_file, "YSize = %d\n", Rend.res_y);
		fprintf (config_file, "FullScreen = %d\n", Rend.fullscreen);
		fprintf (config_file, "ColorDepth = %d\n", Rend.color_depth);
		fprintf (config_file, "ScanLines = %d\n", Rend.scanlines);
		fprintf (config_file, "Filtering = %d\n", Rend.filtering);
		fprintf (config_file, "Blending = %d\n", Rend.blending);
		fprintf (config_file, "Dithering = %d\n", Rend.dithering);
		fprintf (config_file, "ShowFPS = %d\n", Rend.showfps);
		fprintf (config_file, "FrameLimitation = %d\n", Rend.frame_limit);
		fprintf (config_file, "FrameSkipping = %d\n", Rend.frame_skip);
		fprintf (config_file, "FramerateDetection = %d\n", Rend.frame_det);
		fprintf (config_file, "FramerateManual = %d\n", Rend.framerate);
		fprintf (config_file, "TextureType = %d\n", Rend.tex_type);
		fprintf (config_file, "TextureCaching = %d\n", Rend.tex_cache);
		fprintf (config_file, "TurnDisplay = %d\n", Rend.turn_disp);

		// 쓰기 완료, 닫아준다
		fclose (config_file);
	}
	else
	{
#ifdef DEBUG
		fprintf (stderr, "%s writing failure. Permission?\n", cfile_name);
#endif
		result = -1;
	}
		
	g_free (cfile_name);

	return result;


}

/* 키 값 가져오는 인터페이스 */
const gchar * config_keyval_get (const gchar *key)
{
	if (0 == g_ascii_strncasecmp (key, "rompath_dir", 11))
		return Conf.rompath;
	else if (0 == g_ascii_strncasecmp (key, "exec_dir", 8))
		return Conf.exec_path;
	else if (0 == g_ascii_strncasecmp (key, "tmp_dir", 7))
		return Conf.tmp_path;
	
	return NULL;
}

/* 렌더러 설정 값을 가져오는 인터페이스 */
unsigned short int rend_conf_keyval_get (const gchar *key)
{
	if (0 == g_ascii_strncasecmp (key, "XSize", 5))
		return Rend.res_x;
	else if (0 == g_ascii_strncasecmp (key, "YSize", 5))
		return Rend.res_y;
	else if (0 == g_ascii_strncasecmp (key, "FullScreen", 10))
		return Rend.fullscreen;
	else if (0 == g_ascii_strncasecmp (key, "ColorDepth", 10))
		return Rend.color_depth;
	else if (0 == g_ascii_strncasecmp (key, "ScanLines", 9))
		return Rend.scanlines;
	else if (0 == g_ascii_strncasecmp (key, "Filtering", 9))
		return Rend.filtering;
	else if (0 == g_ascii_strncasecmp (key, "Blending", 8))
		return Rend.blending;
	else if (0 == g_ascii_strncasecmp (key, "Dithering", 9))
		return Rend.dithering;
	else if (0 == g_ascii_strncasecmp (key, "ShowFPS", 7))
		return Rend.showfps;
	else if (0 == g_ascii_strncasecmp (key, "FrameLimitation", 15))
		return Rend.frame_limit;
	else if (0 == g_ascii_strncasecmp (key, "FrameSkipping", 13))
		return Rend.frame_skip;
	else if (0 == g_ascii_strncasecmp (key, "FramerateDetection", 18))
		return Rend.frame_det;
	else if (0 == g_ascii_strncasecmp (key, "FramerateManual", 15))
		return Rend.framerate;
	else if (0 == g_ascii_strncasecmp (key, "TextureType", 11))
		return Rend.tex_type;
	else if (0 == g_ascii_strncasecmp (key, "TextureCaching", 14))
		return Rend.tex_cache;
	else if (0 == g_ascii_strncasecmp (key, "TurnDisplay", 11))
		return Rend.turn_disp;

	return 0;

}

/* 윈도우 위치 값을 가져오는 인터페이스 */
int wininfo_keyval_get (const gchar *key)
{
	if (0 == g_ascii_strncasecmp (key, "gzinc_root_x", 12))
		return WinInfo.root_x;
	else if (0 == g_ascii_strncasecmp (key, "gzinc_root_y", 12))
		return WinInfo.root_y;
	else if (0 == g_ascii_strncasecmp (key, "gzinc_width", 11))
		return WinInfo.width;
	else if (0 == g_ascii_strncasecmp (key, "gzinc_height", 12))
		return WinInfo.height;

	return -1;
}

/* 윈도우 값을 한번에 설정하는 인터페이스 */
void wininfo_set_val (int x, int y, int width, int height)
{
	WinInfo.root_x = x;
	WinInfo.root_y = y;
	WinInfo.width = width;
	WinInfo.height = height;

	return;
}

