#include <unistd.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <sys/wait.h>

#include "zinc-exec.h"
#include "configurator.h"
#include "../config-path.h"
#include "about.h"

extern GladeXML *gp_xml_main;		// glade xml

/* Refresh 메뉴 버튼 선택 이벤트 */
void on_menuitem_refresh_activate (GtkMenuItem *menuitem,
										gpointer user_data)
{
	if (NULL != (const gchar *) config_keyval_get ("exec_dir"))
		zinc_gamelist_builder ((const gchar *) config_keyval_get ("exec_dir"));
	else
		zinc_gamelist_builder ("zinc");

	return;
}

/* About(_A) 메뉴 선택시 정보 다이얼로그. */
void on_imagemenuitem_about_activate (GtkMenuItem *menuitem,
										gpointer user_data)
{
	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file ("gzinc.png", NULL);
	if (!pixbuf)
		pixbuf = gdk_pixbuf_new_from_file (INST_PREFIX"/share/pixmaps/gzinc.png", NULL);

	if (pixbuf)
		gtk_show_about_dialog (NULL, "version", VERSION, "license", LICENSE,
		"copyright", COPYRIGHT, "comments", COMMENT, "website", WEBSITE, 
		"logo", pixbuf, NULL);
	else
		gtk_show_about_dialog (NULL, "version", VERSION, "license", LICENSE,
		"copyright", COPYRIGHT, "comments", COMMENT, "website", WEBSITE, NULL);

	gdk_pixbuf_unref (pixbuf);
}

/* 메인 창 Delete 이벤트 */
gboolean on_main_window_delete_event (GtkWidget *widget,
					GdkEvent *event, gpointer user_data)
{
	int root_x = 0, root_y = 0, width = 0, height = 0;
	gtk_window_get_position (GTK_WINDOW (widget), &root_x, &root_y);
	gtk_window_get_size (GTK_WINDOW (widget), &width, &height);

	wininfo_set_val (root_x, root_y, width, height);
	config_write ();
	gtk_main_quit ();

	return FALSE;
}

/* 종료 메뉴아이템 선택 이벤트 */
void on_menuitem_quit_activate (GtkMenuItem *menuitem,
								gpointer user_data)
{
	int root_x = 0, root_y = 0, width = 0, height = 0;
	GtkWidget *main_window = glade_xml_get_widget (gp_xml_main,
													"main_window");

	gtk_window_get_position (GTK_WINDOW (main_window), &root_x, &root_y);
	gtk_window_get_size (GTK_WINDOW (main_window), &width, &height);
	
	wininfo_set_val (root_x, root_y, width, height);
	config_write ();
	gtk_main_quit ();

	return;
}

/* 설정 다이얼로그 창 닫기 이벤트 */
gboolean on_pref_dialog_delete_event (GtkWidget *widget,
					GdkEvent *event, gpointer user_data)
{
	/* gtk_widget_hide_on_delete ()를 사용해야 할 경우에는 TRUE를
	 * 리턴해서 delete event로 인해 창이 destroy 되지 않도록 한다 */
	gtk_widget_hide_on_delete (widget);
	return TRUE;
}

/* 렌더러 설정 다이얼로그 창 닫기 이벤트 */
gboolean on_rend_dialog_delete_event (GtkWidget *widget,
					GdkEvent *event, gpointer user_data)
{
	/* gtk_widget_hide_on_delete ()를 사용해야 할 경우에는 TRUE를
	 * 리턴해서 delete event로 인해 창이 destroy 되지 않도록 한다 */
	gtk_widget_hide_on_delete (widget);
	return TRUE;
}

/* 설정 메뉴 아이템 선택 이벤트 */
void on_menuitem_pref_activate (GtkMenuItem *menuitem,
							gpointer user_data)
{
	GtkWidget *pref_dialog = glade_xml_get_widget (gp_xml_main, "pref_dialog");

	gtk_widget_show (pref_dialog);
}

/* 렌더러 설정 메뉴 아이템 선택 이벤트 */
void on_menuitem_rend_activate (GtkMenuItem *menuitem,
							gpointer user_data)
{
	GtkWidget *rend_dialog = glade_xml_get_widget (gp_xml_main, "rend_dialog");

	gtk_widget_show (rend_dialog);
}

/* 설정 다이얼로그 확인 버튼 클릭 이벤트 */
void on_pref_ok_button_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *zinc_exec_entry = glade_xml_get_widget (gp_xml_main,
						"zinc_exec_entry");
	GtkWidget *rom_dir_entry = glade_xml_get_widget (gp_xml_main,
						"rom_dir_entry");
	GtkWidget *tmp_dir_entry = glade_xml_get_widget (gp_xml_main,
						"tmp_dir_entry");
	GtkWidget *pref_dialog = glade_xml_get_widget (gp_xml_main, 
						"pref_dialog");

	if (0 < g_utf8_strlen (gtk_entry_get_text (GTK_ENTRY(rom_dir_entry)), 400))
		config_keyval_set ("rompath_dir", gtk_entry_get_text (GTK_ENTRY (rom_dir_entry)));
	
	if (0 < g_utf8_strlen (gtk_entry_get_text (GTK_ENTRY (zinc_exec_entry)), 400))
		config_keyval_set ("exec_dir", gtk_entry_get_text (GTK_ENTRY (zinc_exec_entry)));
	
	if (0 < g_utf8_strlen (gtk_entry_get_text (GTK_ENTRY(tmp_dir_entry)), 400))
		config_keyval_set ("tmp_dir", gtk_entry_get_text (GTK_ENTRY (tmp_dir_entry)));

	// 설정 저장.
	if (0 != config_write ())
		fprintf (stderr, "config_write() failure!\n");

	gtk_widget_hide (pref_dialog);
}

/* 설정 다이얼로그 취소 버튼 클릭 이벤트 */
void on_pref_cancel_button_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *pref_dialog = glade_xml_get_widget (gp_xml_main, "pref_dialog");
	gtk_widget_hide (pref_dialog);
}

/* 렌더러 설정 다이얼로그 적용 버튼 클릭 이벤트 */
void on_rend_ok_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *widget = NULL;
	char tmp[8];
	
	widget = glade_xml_get_widget (gp_xml_main, "rend_dialog");
	gtk_widget_hide (widget);

	// 가로 해상도
	widget = glade_xml_get_widget (gp_xml_main, "entry_res_x");
	rend_conf_keyval_set ("XSize", gtk_entry_get_text (GTK_ENTRY (widget)));
	
	// 세로 해상도
	widget = glade_xml_get_widget (gp_xml_main, "entry_res_y");
	rend_conf_keyval_set ("YSize", gtk_entry_get_text (GTK_ENTRY (widget)));

	// Fullscreen mode
	widget = glade_xml_get_widget (gp_xml_main, "cb_fullscreen");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("FullScreen", "1");
	else
		rend_conf_keyval_set ("FullScreen", "0");

	// Color Depth
	widget = glade_xml_get_widget (gp_xml_main, "rb_16bit");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("ColorDepth", "16");
	else
		rend_conf_keyval_set ("ColorDepth", "32");
	
	// Scanlines
	widget = glade_xml_get_widget (gp_xml_main, "combobox_scanline");
	g_snprintf (tmp, 8, "%d", (unsigned short int) gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
	rend_conf_keyval_set ("ScanLines", tmp);

	// filtering 설정
	widget = glade_xml_get_widget (gp_xml_main, "rb_tf_0");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("Filtering", "0");
	widget = glade_xml_get_widget (gp_xml_main, "rb_tf_1");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("Filtering", "1");
	widget = glade_xml_get_widget (gp_xml_main, "rb_tf_2");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("Filtering", "2");
	widget = glade_xml_get_widget (gp_xml_main, "rb_tf_3");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("Filtering", "3");
	
	// blending 설정
	widget = glade_xml_get_widget (gp_xml_main, "cb_ecb");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("Blending", "1");
	else
		rend_conf_keyval_set ("Blending", "0");
	
	// dithering 설정
	widget = glade_xml_get_widget (gp_xml_main, "cb_dither");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("Dithering", "1");
	else
		rend_conf_keyval_set ("Dithering", "0");
	
	// showfps 설정
	widget = glade_xml_get_widget (gp_xml_main, "cb_showfps");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("ShowFPS", "1");
	else
		rend_conf_keyval_set ("ShowFPS", "0");

	// frame limitation 설정
	widget = glade_xml_get_widget (gp_xml_main, "cb_framelimit");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("FrameLimitation", "1");
	else
		rend_conf_keyval_set ("FrameLimitation", "0");
	
	// frame skipping 설정
	widget = glade_xml_get_widget (gp_xml_main, "cb_frameskip");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("FrameSkipping", "1");
	else
		rend_conf_keyval_set ("FrameSkipping", "0");
	
	// Auto Framerate Detection 설정
	widget = glade_xml_get_widget (gp_xml_main, "cb_afrd");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("FramerateDetection", "1");
	else
		rend_conf_keyval_set ("FramerateDetection", "0");
	
	// Manual Framerate 설정
	widget = glade_xml_get_widget (gp_xml_main, "manual_frate");
	g_snprintf (tmp, 8, "%d", (unsigned short int) gtk_range_get_value (GTK_RANGE (widget)));

	// Texture type 설정
	widget = glade_xml_get_widget (gp_xml_main, "rb_tex_3");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TextureType", "3");
	widget = glade_xml_get_widget (gp_xml_main, "rb_tex_2");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TextureType", "2");
	widget = glade_xml_get_widget (gp_xml_main, "rb_tex_1");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TextureType", "1");
	widget = glade_xml_get_widget (gp_xml_main, "rb_tex_0");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TextureType", "0");

	// Texture cache 설정
	widget = glade_xml_get_widget (gp_xml_main, "rb_cache_0");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TextureCaching", "0");
	widget = glade_xml_get_widget (gp_xml_main, "rb_cache_1");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TextureCaching", "1");
	widget = glade_xml_get_widget (gp_xml_main, "rb_cache_2");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TextureCaching", "2");
	
	// Turn Display 설정
	widget = glade_xml_get_widget (gp_xml_main, "rb_rotate_0");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TurnDisplay", "0");
	widget = glade_xml_get_widget (gp_xml_main, "rb_rotate_1");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TurnDisplay", "1");
	widget = glade_xml_get_widget (gp_xml_main, "rb_rotate_2");
	if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
		rend_conf_keyval_set ("TurnDisplay", "2");

	rend_conf_write ();
	return;
}

/* 렌더러 설정 다이얼로그 취소 버튼 클릭 이벤트 */
void on_rend_cancel_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *rend_dialog = glade_xml_get_widget (gp_xml_main, "rend_dialog");
	gtk_widget_hide (rend_dialog);
}

/* 트리뷰 커서 선택 이벤트 */
void on_gs_treeview_cursor_changed (GtkTreeView *treeview,
											gpointer user_data)
{
	gchar temp[100];

	GtkTreeIter t_gs_iter;
	GtkTreeSelection *p_gs_ts = gtk_tree_view_get_selection
									((GtkTreeView *) treeview);
	GtkTreeModel *p_gs_tm = NULL;
	gchar *romname = NULL;
	
	/* 현재 선택된 모델과 iterator를 구하고 게임 롬 string 값을 가져온다 */
	gtk_tree_selection_get_selected (p_gs_ts, &p_gs_tm, &t_gs_iter);
	gtk_tree_model_get (p_gs_tm, &t_gs_iter, GS_GAME_ROM, &romname, -1); 

	GtkImage *artwork = (GtkImage *) glade_xml_get_widget (gp_xml_main, 
							"artwork_image");

	g_snprintf (temp, 100, INST_PREFIX"/share/gzinc/artworks/%s.png", romname);
	
	if (0 != access (temp, R_OK)) {
		g_snprintf (temp, 100, "../artworks/%s.png", romname);
	}
	
	GdkPixbuf *aw_pixbuf = gdk_pixbuf_new_from_file (temp, NULL);
	GdkPixbuf *aw_scaled = NULL;
	
	if (aw_pixbuf != NULL) {
		aw_scaled = gdk_pixbuf_scale_simple (aw_pixbuf, 320, 240, 
							GDK_INTERP_BILINEAR);
		gtk_image_set_from_pixbuf (artwork, aw_scaled);

		g_object_unref (aw_pixbuf);
		g_object_unref (aw_scaled);
	}
	else {
		gtk_image_set_from_stock (artwork, "gtk-missing-image", 
						GTK_ICON_SIZE_LARGE_TOOLBAR);
	}
}

/* 해당 게임의 열을 선택시: 게임 실행 */
void on_gs_treeview_row_activated (GtkTreeView *treeview,
		GtkTreePath *path, GtkTreeViewColumn *column,
					gpointer user_data)
{
	GtkWidget *main_window = glade_xml_get_widget (gp_xml_main,
						"main_window");

	GtkTreeIter t_gs_iter;
	GtkTreeSelection *p_gs_ts = gtk_tree_view_get_selection (
						(GtkTreeView *) treeview);

	GtkTreeModel *p_gs_tm = NULL;
	gchar *rom_num = NULL;
	gint root_x, root_y;


	/* 현재 선택된 모델과 iterator를 구하고 게임 번호 값을 가져온다 */
	gtk_tree_selection_get_selected (p_gs_ts, &p_gs_tm, &t_gs_iter);
	gtk_tree_model_get (p_gs_tm, &t_gs_iter, GS_GAME_NUM, &rom_num, -1); 

	// 윈도우의 위치와 정보를 받고 숨긴다
	gtk_window_get_position (GTK_WINDOW (main_window), &root_x, &root_y);
	gdk_window_hide (GDK_WINDOW (main_window->window));
	gdk_window_process_all_updates ();
	
	// zinc 실행
	exec_zinc_game (rom_num);

	// 실행 후에는 다시 원래의 윈도우로 복귀 시킨다
	gdk_window_show (GDK_WINDOW (main_window->window));
	gtk_window_move (GTK_WINDOW (main_window), root_x, root_y);

	return;
}

/* zinc 실행파일 위치 버튼 클릭 이벤트: GtkFileChooserDialog 띄움 */
void on_zinc_find_button_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *z_dialog = NULL;
	GtkWidget *parent = glade_xml_get_widget (gp_xml_main, "pref_dialog");

	z_dialog = gtk_file_chooser_dialog_new ("Open ZiNc Executables",
						GTK_WINDOW (parent),
						GTK_FILE_CHOOSER_ACTION_OPEN,
						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
						NULL);
	
	if (GTK_RESPONSE_ACCEPT == gtk_dialog_run (GTK_DIALOG (z_dialog)))
	{
		gchar *filename = NULL;
		GtkWidget *z_entry = glade_xml_get_widget (gp_xml_main, "zinc_exec_entry");
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (z_dialog));
		gtk_entry_set_text (GTK_ENTRY (z_entry), filename);
		g_free (filename);
		filename = NULL;
	}

	gtk_widget_destroy (z_dialog);
	return;
}

/* 롬 디렉토리 선택 이벤트 */
void on_rom_dir_button_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *r_dialog = NULL;
	GtkWidget *parent = glade_xml_get_widget (gp_xml_main, "pref_dialog");

	r_dialog = gtk_file_chooser_dialog_new ("Select ZiNc ROM Directory",
									GTK_WINDOW (parent),
									GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
									GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
									GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
									NULL);
	
	if (GTK_RESPONSE_ACCEPT == gtk_dialog_run (GTK_DIALOG (r_dialog)))
	{
		gchar *pathname = NULL;
		GtkWidget *r_entry = glade_xml_get_widget (gp_xml_main, "rom_dir_entry");
		pathname = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (r_dialog));
		gtk_entry_set_text (GTK_ENTRY (r_entry), pathname);
		g_free (pathname);
		pathname = NULL;
	}

	gtk_widget_destroy (r_dialog);
}

/* 임시 디렉토리 선택 이벤트 */
void on_tmp_dir_button_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *t_dialog = NULL;
	GtkWidget *parent = glade_xml_get_widget (gp_xml_main, "pref_dialog");

	t_dialog = gtk_file_chooser_dialog_new ("Select Tmp Directory",
									GTK_WINDOW (parent),
									GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
									GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
									GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
									NULL);
	
	if (GTK_RESPONSE_ACCEPT == gtk_dialog_run (GTK_DIALOG (t_dialog)))
	{
		gchar *pathname = NULL;
		GtkWidget *t_entry = glade_xml_get_widget (gp_xml_main, "tmp_dir_entry");
		pathname = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (t_dialog));
		gtk_entry_set_text (GTK_ENTRY (t_entry), pathname);
		g_free (pathname);
		pathname = NULL;
	}

	gtk_widget_destroy (t_dialog);

}
