/* Zinc - gZinc Connector */

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "gzinc-types.h"
#include "zinc-exec.h"
#include "configurator.h"

extern GladeXML *gp_xml_main;

/* Game List를 담을 Tree */
static GtkTreeStore *p_gs_store = NULL;

/* Game List TreeStore에 아이템을 저장하기 위한 매크로 대용 인라인 함수. */
inline static void gs_treestore_additem (GtkTreeIter *iter,
		const char *s_num, const char *s_title,
		const char *s_romfile, const char *s_parent, const char *s_bios)
{
	gtk_tree_store_append (p_gs_store, iter, NULL);
	gtk_tree_store_set (p_gs_store, iter,
					GS_GAME_NUM,	s_num,
					GS_GAME_TITLE,	s_title,
					GS_GAME_ROM,	s_romfile,
					GS_PARENT_ROM,	s_parent,
					GS_BIOS,		s_bios, 
					-1);
}


/* Init: Game List Building Procedures */
int zinc_gamelist_builder (const char *zinc_executable)
{
	FILE *f_pipe;
	char buf[256];
	unsigned short count = 0;
	char *token, *saveptr;

	gameinfo tmp;

	GtkWidget *p_gs_tree = glade_xml_get_widget (gp_xml_main, "gs_treeview");
	GtkTreeIter gs_iter;
	GtkTreeModel *p_gs_model = NULL;

	p_gs_store = gtk_tree_store_new (NUM_COLS_GS,
		G_TYPE_STRING,	G_TYPE_STRING,	G_TYPE_STRING,
		G_TYPE_STRING,	G_TYPE_STRING);

	gchar *tmp_command = NULL;
	tmp_command = g_strdup_printf ("%s --list-games", zinc_executable);

	f_pipe = popen (tmp_command, "r");
	g_free (tmp_command);

	if (f_pipe == NULL) {
		perror ("popen() error : ");
		return -1;
	}

	while (fgets (buf, 256, f_pipe) != NULL) {
		if (count == 0) {
			token = (char *) strtok_r (buf, " ", &saveptr);
			if (0 == strcmp ("ZiNc", token)) {
				token = (char *) strtok_r (NULL, " ", &saveptr);
#ifdef DEBUG
				fprintf (stderr, "ZiNc Executable Found, version %s\n\n", token);
#endif
				count++;
				continue;
			}
			else {
				fprintf (stderr, "ZiNc Executable Not Found!\n");
				return -1;
			}
		}
		else {
			token = (char *) strtok_r (buf, " ", &saveptr);
			snprintf (tmp.num, 4, "%s", token);
#ifdef DEBUG
			fprintf (stderr, "num: %s\n", token);
#endif
			token = (char *) strtok_r (NULL, "[", &saveptr);
			snprintf (tmp.name, 128, "%s", token);
#ifdef DEBUG
			fprintf (stderr, "title: %s\n", token);
#endif
			token = (char *) strtok_r (NULL, ",]", &saveptr);
			snprintf (tmp.romfilename, 10, "%s", token);
#ifdef DEBUG
			fprintf (stderr, "romname: %s\n", token);
#endif
			token = (char *) strtok_r (NULL, " ,]\n", &saveptr);
			if (token == NULL) {
				gs_treestore_additem (&gs_iter, tmp.num, tmp.name,
								tmp.romfilename, NULL, NULL);
				continue;
			}
			else if (0 == strcmp ("parent", token)) {
				token = (char *) strtok_r (NULL, " ,]\n", &saveptr);
				snprintf (tmp.parentname, 10, "%s", token);
#ifdef DEBUG
				fprintf (stderr, "parent needed : %s\n", token);
#endif
			}
			else if (0 == strcmp ("BIOS", token)) {
				token = (char *) strtok_r (NULL, " ,]\n", &saveptr);
				snprintf (tmp.biosname, 10, "%s", token);
#ifdef DEBUG
				fprintf (stderr, "BIOS needed : %s\n\n", token);
#endif
				gs_treestore_additem (&gs_iter, tmp.num, tmp.name,
								tmp.romfilename, NULL, tmp.biosname);
				continue;
			}

			token = (char *) strtok_r (NULL, " ,]\n", &saveptr);
			if (token == NULL) {
#ifdef DEBUG
				fprintf (stderr, "\n");
#endif
				gs_treestore_additem (&gs_iter, tmp.num, tmp.name,
								tmp.romfilename, tmp.parentname, NULL);
				continue;
			}
			else if (0 == strcmp ("BIOS", token)) {
				token = (char *) strtok_r (NULL, " ,]\n", &saveptr);
				snprintf (tmp.biosname, 10, "%s", token);
#ifdef DEBUG
				fprintf (stderr, "BIOS needed : %s\n\n", token);
#endif
				gs_treestore_additem (&gs_iter, tmp.num, tmp.name,
								tmp.romfilename, tmp.parentname, tmp.biosname);
				continue;
			}
		}
	}

	p_gs_model = GTK_TREE_MODEL (p_gs_store);
	gtk_tree_view_set_model (GTK_TREE_VIEW (p_gs_tree), p_gs_model);
	g_object_unref (p_gs_model);

	pclose (f_pipe);
	
	return 0;
}

/* romnum을 받아서 게임을 실행. */
int exec_zinc_game (const gchar *romnum)
{
	/* FIXME :
	 *  1. tmp_dir을 확인, 내부에 (tmp_dir)/gzinc/cfg 디렉토리를 생성.
	 *  2. 보관된 롬 이름과 일치하는 cfg를 restore. 
	 */
	gchar *tmp_argv = NULL;
	
	if ((NULL != config_keyval_get ("exec_dir")) && (NULL != config_keyval_get ("rompath_dir")))
		tmp_argv = g_strdup_printf ("%s %s --roms-directory=%s --use-renderer-cfg-file=%s/.gzinc_rend", config_keyval_get ("exec_dir"), romnum, config_keyval_get ("rompath_dir"), getenv ("HOME"));
	else if (NULL != config_keyval_get ("rompath_dir"))
		tmp_argv = g_strdup_printf ("zinc %s --roms-directory=%s --use-renderer-cfg-file=%s/.gzinc_rend", romnum, config_keyval_get ("rompath_dir"), getenv ("HOME"));
	else
	{
		fprintf (stderr, "rompath NOT set, abort execution!\n");
		return;
	}

	/* FIXME :
	 *  1.	g_spawn_sync () 함수로 교체해야 함. 출력을 받아서
	 *		실행 실패시 이유를 보여줄 수 있어야 함. 
	 *  2.	역시 마찬가지로 working_directory를 tmp_dir 을 사용하게 해야 함
	 */
	g_spawn_command_line_sync (tmp_argv, NULL, NULL, NULL, NULL);
	
	g_free (tmp_argv);
	tmp_argv = NULL;

	/* FIXME :
	 *  1. 실행 완료시 새로 갱신된 (tmp_dir)/gzinc/cfg/ 디렉토리내의 설정을
	 *     저장하여 다음에 계속 이어서 쓸 수 있도록 해야 함.
	 */
}
